import { __decorate } from "tslib";
import { attr, observable } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { SNACKBAR_TYPE_DEFAULT, SNACKBAR_CLOSE_TIMEOUT, SNACKBAR_EVENTS_NAMES, } from './snackbar.config';
import { foundationSnackbarStyles as styles } from './snackbar.styles';
import { foundationSnackbarTemplate as template } from './snackbar.template';
export const foundationSnackbarShadowOptions = {
    mode: 'open',
    delegatesFocus: true,
};
/**
 * @tagname %%prefix%%-snackbar
 */
export class Snackbar extends FoundationElement {
    constructor() {
        super(...arguments);
        this.deleteAnimation = '';
        this.closeTimeout = SNACKBAR_CLOSE_TIMEOUT;
        this.autoClose = false;
        this.type = SNACKBAR_TYPE_DEFAULT;
        this.dismissEventListener = this.handleDissmis.bind(this);
    }
    connectedCallback() {
        super.connectedCallback();
        if (this.autoClose) {
            this.timeoutRef = setTimeout(() => {
                this.handleDissmis(true);
            }, this.closeTimeout);
        }
        this.addEventListener(SNACKBAR_EVENTS_NAMES.DISMISS, this.dismissEventListener);
    }
    errorChanged(oldValue, newValue) {
        var _a, _b, _c;
        this.type = (_c = (_b = (_a = newValue === null || newValue === void 0 ? void 0 : newValue.config) === null || _a === void 0 ? void 0 : _a.snackbar) === null || _b === void 0 ? void 0 : _b.type) !== null && _c !== void 0 ? _c : SNACKBAR_TYPE_DEFAULT;
    }
    notificationChanged(oldValue, newValue) {
        var _a, _b, _c;
        this.type = (_c = (_b = (_a = newValue === null || newValue === void 0 ? void 0 : newValue.config) === null || _a === void 0 ? void 0 : _a.snackbar) === null || _b === void 0 ? void 0 : _b.type) !== null && _c !== void 0 ? _c : SNACKBAR_TYPE_DEFAULT;
    }
    /**
     * Helper for remove snackbar from display with animation
     */
    handleDissmis(autoDissmis = false) {
        this.deleteAnimation = 'snackbar-delete';
        this.snackbar.onanimationend = () => {
            this.dismiss();
            if (autoDissmis) {
                this.dispatchEvent(new CustomEvent(SNACKBAR_EVENTS_NAMES.AUTO_DISMISS, {
                    bubbles: true,
                    composed: true,
                    detail: this.identifier,
                }));
            }
        };
    }
    /**
     * Remove the snackbar from display.
     */
    dismiss() {
        this.remove();
        if (this.timeoutRef) {
            clearTimeout(this.timeoutRef);
        }
    }
    disconnectedCallback() {
        super.disconnectedCallback();
        this.removeEventListener(SNACKBAR_EVENTS_NAMES.DISMISS, this.dismissEventListener);
        if (this.timeoutRef) {
            clearTimeout(this.timeoutRef);
        }
    }
}
__decorate([
    observable
], Snackbar.prototype, "error", void 0);
__decorate([
    observable
], Snackbar.prototype, "notification", void 0);
__decorate([
    observable
], Snackbar.prototype, "deleteAnimation", void 0);
__decorate([
    attr({ attribute: 'identifier' })
], Snackbar.prototype, "identifier", void 0);
__decorate([
    attr({ attribute: 'close-timeout' })
], Snackbar.prototype, "closeTimeout", void 0);
__decorate([
    attr({ attribute: 'auto-close', mode: 'boolean' })
], Snackbar.prototype, "autoClose", void 0);
__decorate([
    attr({ attribute: 'type' })
], Snackbar.prototype, "type", void 0);
/**
 * The Foundation Snackbar
 *
 * @public
 * @remarks
 * HTML Element: \<foundation-snackbar\>
 */
export const foundationSnackbar = Snackbar.compose({
    baseName: 'snackbar',
    styles,
    template,
    shadowOptions: foundationSnackbarShadowOptions,
});
